@@ -1,9 +1,8 @@
-__version__ = '0.4.0'
-
 from .core import *
 from .io import *
 from .array_functions import *
-from .reference_values import *
 
 from . import tb
-from . import ep
\ No newline at end of file
+from .reference_values import ReferenceValues as _RefVal
+
+refval = _RefVal()
