@@ -1,23 +1,23 @@
 import os as _os
 from isopy import core
-
 import csv as csv
 import datetime as dt
 import numpy as np
 import chardet
 import openpyxl
+import pyperclip
 from openpyxl import load_workbook
-import functools
 import itertools
 import io
-import pkg_resources as _pkgr
-from typing import TypeVar, Union, Optional, Any, NoReturn, NamedTuple, Literal, Callable
-from collections.abc import Sequence, Iterable
+import numpy as np
 
-__all__ = ['read_exp', 'read_csv', 'write_csv', 'read_xlsx', 'write_xlsx']
+
+__all__ = ['read_exp', 'read_csv', 'write_csv', 'read_xlsx', 'write_xlsx', 'read_clipboard',
+           'array_from_csv', 'array_from_xlsx', 'array_from_clipboard']
 
 import isopy.checks
 
+
 ################
 ### read exp ###
 ################
@@ -25,13 +25,11 @@ class NeptuneData:
     """
     Container for the data returned by ``read_exp``.
     """
-    def __init__(self, info, cycle, time, isotope_data, ratio_data, other_data):
+    def __init__(self, info, cycle, time, measurements):
         self.info = info
         self.cycle = cycle
         self.time = time
-        self.isotope_data = isotope_data
-        self.ratio_data = ratio_data
-        self.other_data = other_data
+        self.measurements = measurements
 
 def read_exp(filename, rename = None) -> NeptuneData:
     """
@@ -39,8 +37,9 @@ def read_exp(filename, rename = None) -> NeptuneData:
 
     Parameters
     ----------
-    filename : str, bytes
+    filename : str, bytes, StringIO, BytesIO
         Path for file to be opened. Alternatively a file like byte string can be supplied.
+        Also accepts file like objects.
     rename : dict, Callable, Optional
         For renaming keys in the analysed data. Useful for cases when the key is the mass rather
         than the isotope measured. If a dictionary is passed then every key present in the
@@ -56,21 +55,34 @@ def read_exp(filename, rename = None) -> NeptuneData:
         * info - Dictionary containing the metadata included at the beginning of the file.
         * cycle - A list containing the cycle number for each measurement.
         * time - A list containing datetime objects for each measurement.
-        * isotope_data - A dictionary containing an isotope array, with all the data with an isotope keystring, for each line measured. Static measurements are always given as line ``1``.
-        * ratio_data - A dictionary containing an ratio array, with all the data with an ratio keystring, for each line measured. Static measurements are always given as line ``1``.
-        * other_data - A dictionary containing an general array, with all leftover data, for each line measured. Static measurements are always given as line ``1``.
+        * measurements - An dictionary containing an an isopy array with the values in for each line measured. Static measurements are always given as line ``1``.
+         to extract e.g only the isotope data from the measurement use ``neptune_data.measurement[line].copy(flavour_eq='isotope')``.
     """
     information = {}
 
-    if type(filename) is not bytes:
-        fileio = open(filename, 'rb')
-        file = fileio.read()
-        fileio.close()
-    else:
+    # If filename is a string load the files.
+    if type(filename) is str:
+        with open(filename, 'rb') as fileio:
+            file = fileio.read()
+    elif type(filename) is bytes:
         file = filename
+    elif type(filename) is io.BytesIO:
+        filename.seek(0)
+        file = filename.read()
+    elif type(filename) is str:
+        file = filename
+    elif type(filename) is io.StringIO:
+        filename.seek(0)
+        file = filename.read()
+    else:
+        raise TypeError('filename is of unknown type')
 
-    encoding = chardet.detect(file).get('encoding')
-    file = file.decode(encoding)
+    if type(file is bytes):
+        # find the files encoding
+        encoding = chardet.detect(file).get('encoding')
+
+        # Decode the bytes into string.
+        file = file.decode(encoding)
 
     dialect = csv.Sniffer().sniff(file)
 
@@ -93,58 +105,48 @@ def read_exp(filename, rename = None) -> NeptuneData:
     else:
         raise TypeError('rename must be a dict or a callable function')
 
+    data.pop("", None)
     cycle = np.array(data.pop('Cycle'), dtype ='int')
     time = data.pop('Time')
-    time =[dt.datetime.strptime(time[i], '%H:%M:%S:%f') for i in range(len(time))]
-    isotope_data = {}
-    ratio_data = {}
-    other_data = {}
+    try:
+        time =[dt.datetime.strptime(time[i], '%H:%M:%S:%f') for i in range(len(time))]
+    except ValueError:
+        try:
+            time = [dt.datetime.fromtimestamp(time[i]).strftime('%H:%M:%S:%f') for i in range(len(time))]
+        except:
+            time = [dt.datetime.fromtimestamp(0).strftime('%H:%M:%S:%f') for i in range(len(time))]
+    measurements = {}
 
     for key in data:
         if ":" in key:
-            line, text = key.split(":", 1)
+            line, keystring = key.split(":", 1)
             line = int(line)
         else:
             line = 1
-            text = key
-        text = renamer(text)
+            keystring = key
+        keystring = renamer(keystring)
+        measurements.setdefault(line, {})[isopy.keystring(keystring)] = data[key]
 
-        try:
-            text = isopy.IsotopeKeyString(text)
-        except:
-            pass
-        else:
-            isotope_data.setdefault(line, {})[text] == data[key]
-            continue
 
-        try:
-            text = isopy.RatioKeyString(text)
-        except:
-            pass
-        else:
-            ratio_data.setdefault(line, {})[text] == data[key]
-            continue
-
-        other_data.setdefault(line, {})[text] = data[key]
-
-    isotope_data = {line: isopy.core.IsotopeArray(isotope_data[line]) for line in isotope_data}
-    ratio_data = {line: isopy.core.RatioArray(ratio_data[line]) for line in ratio_data}
-    other_data = {line: isopy.core.GeneralArray(other_data[line]) for line in other_data}
-    return NeptuneData(information, cycle, time, isotope_data, ratio_data, other_data)
+    measurements = {line: isopy.asarray(measurements[line]) for line in measurements}
+    return NeptuneData(information, cycle, time, measurements)
 
 ######################
 ### read/write CSV ###
 ######################
 def read_csv(filename, comment_symbol ='#', keys_in = 'c',
              float_preferred = False, encoding = None,
-             dialect = None ):
+             dialect = None):
     """
     Load data from a csv file.
 
+
+
     Parameters
     ----------
-    filename : str, bytes
+    filename : str, bytes, StringIO, BytesIO
         Path for file to be opened. Alternatively a file like byte string can be supplied.
+        Also accepts file like objects.
     comment_symbol : str, Default = '#'
         Rows starting with this string will be ignored.
     keys_in : {'c', 'r', None}, Default = 'c'
@@ -170,19 +172,29 @@ def read_csv(filename, comment_symbol ='#', keys_in = 'c',
     dialect = isopy.checks.check_type('dialect', dialect, str, allow_none=True)
 
     #If filename is a string load the files.
-    if type(filename) is not bytes:
-        fileio = open(filename, 'rb')
-        file = fileio.read()
-        fileio.close()
-    else:
+    if type(filename) is str:
+        with open(filename, 'rb') as fileio:
+            file = fileio.read()
+    elif type(filename) is bytes:
         file = filename
+    elif type(filename) is io.BytesIO:
+        filename.seek(0)
+        file = filename.read()
+    elif type(filename) is str:
+        file = filename
+    elif type(filename) is io.StringIO:
+        filename.seek(0)
+        file = filename.read()
+    else:
+        raise TypeError('filename is of unknown type')
 
-    #find the files encoding
-    if encoding is None:
-        encoding = chardet.detect(file).get('encoding')
+    if type(file is bytes):
+        #find the files encoding
+        if encoding is None:
+            encoding = chardet.detect(file).get('encoding')
 
-    #Decode the bytes into string.
-    file = file.decode(encoding)
+        #Decode the bytes into string.
+        file = file.decode(encoding)
 
     #find the csv dialect
     if dialect is None:
@@ -198,19 +210,51 @@ def read_csv(filename, comment_symbol ='#', keys_in = 'c',
             # All the columns in this row are empty. Ignore it
             continue
 
-        if comment_symbol is not None and row[0].startswith(comment_symbol):
+        if comment_symbol is not None and row[0][:len(comment_symbol)] == comment_symbol: #startswith(comment_symbol):
             # This is a comment so ignore it.
             continue
 
         if keys_in == 'c':
-            return _read_csv_ckeys(row_data, csv_reader, comment_symbol, float_prefered=float_preferred)
+            data =  _read_csv_ckeys(row_data, csv_reader, comment_symbol, float_prefered=float_preferred)
+            data.pop("", None)
+            return data
         elif keys_in == 'r':
-            return _read_csv_rkeys(row_data, csv_reader, comment_symbol, float_prefered=float_preferred)
+            data =  _read_csv_rkeys(row_data, csv_reader, comment_symbol, float_prefered=float_preferred)
+            data.pop("", None)
+            return data
         elif keys_in is None:
             return _read_csv_nokeys(row_data, csv_reader, comment_symbol, float_prefered=float_preferred)
         else:
             raise ValueError(f'Unknown value for "keys_in" {keys_in}')
 
+def read_clipboard(comment_symbol ='#', keys_in = 'c',
+             float_preferred = False, dialect = None):
+    """
+    Load data from values in the clipboard.
+
+    Parameters
+    ----------
+    comment_symbol : str, Default = '#'
+        Rows starting with this string will be ignored.
+    keys_in : {'c', 'r', None}, Default = 'c'
+        If keys are given as the first value in each column pass ``c``. If keys are given as the
+        first value in each row pass ``r``. If there are no keys pass ``None``.
+    float_preferred : bool, Default = False
+        If `True` all values will be converted to float if possible. If conversion fails the
+        value will be left as a string.
+    dialect
+        Dialect of the values in the clipboard. If None the dialect will be guessed from the values.
+
+    Returns
+    -------
+    data : dict or list
+        Returns a dictionary for data with keys otherwise a list.
+    """
+    data = pyperclip.paste()
+    data = data.encode('UTF-8')
+    return read_csv(data, encoding='UTF-8', comment_symbol=comment_symbol, keys_in=keys_in,
+                    float_preferred=float_preferred, dialect=dialect)
+
 
 def _read_csv_ckeys(first_row, reader, comment_symbol=None, termination_symbol=None, float_prefered = False):
     keys = first_row  # First row contains the the name of each column
@@ -220,11 +264,11 @@ def _read_csv_ckeys(first_row, reader, comment_symbol=None, termination_symbol=N
     for i, row in enumerate(reader):
         row = [r.strip() for r in row]
 
-        if termination_symbol is not None and row[0].startswith(termination_symbol):
+        if termination_symbol is not None and row[0][:len(termination_symbol)] == termination_symbol: #startswith(termination_symbol):
             # Stop reading data if we find this string at the beginning of a row
             break
 
-        if comment_symbol is not None and row[0].startswith(comment_symbol):
+        if comment_symbol is not None and row[0][:len(comment_symbol)] == comment_symbol: #.startswith(comment_symbol):
             # Row is a comment, ignore
             continue
 
@@ -259,11 +303,11 @@ def _read_csv_rkeys(first_row, reader, comment_symbol=None, termination_symbol=N
     for i, row in enumerate(itertools.chain([first_row], reader)):
         row = [r.strip() for r in row]
 
-        if termination_symbol is not None and row[0].startswith(termination_symbol):
+        if termination_symbol is not None and row[0][:len(termination_symbol)] == termination_symbol: #.startswith(termination_symbol):
             # Stop reading data if we find this string at the beginning of a row
             break
 
-        if comment_symbol is not None and row[0].startswith(comment_symbol):
+        if comment_symbol is not None and row[0][:len(comment_symbol)] == comment_symbol: #.startswith(comment_symbol):
             # Row is a comment, ignore
             continue
 
@@ -299,11 +343,11 @@ def _read_csv_nokeys(first_row, reader, comment_symbol=None, termination_symbol=
     for i, row in enumerate(itertools.chain([first_row], reader)):
         row = [r.strip() for r in row]
 
-        if termination_symbol is not None and row[0].startswith(termination_symbol):
+        if termination_symbol is not None and row[0][:len(termination_symbol)] == termination_symbol: #.startswith(termination_symbol):
             # Stop reading data if we find this string at the beginning of a row
             break
 
-        if comment_symbol is not None and row[0].startswith(comment_symbol):
+        if comment_symbol is not None and row[0][:len(comment_symbol)] == comment_symbol: #.startswith(comment_symbol):
             # Row is a comment, ignore
             continue
 
@@ -338,8 +382,9 @@ def write_csv(filename, data, comments=None, keys_in='c',
 
     Parameters
     ----------
-    filename : str
-        Path/name of the csv file to be created. Any existing file with the same path/name will be overwritten.
+    filename : str, StringIO, BytesIO
+        Path/name of the csv file to be created. Any existing file with the same path/name will be
+        over written. Also accepts file like objects.
     data : isopy_array_like, numpy_array_like
         Data to be saved in the array.
     comments : str, Sequence[str], Optional
@@ -352,7 +397,7 @@ def write_csv(filename, data, comments=None, keys_in='c',
     comment_symbol : str, Default = '#'
         This string will precede any comments at the beginning of the file.
     """
-    filename = isopy.checks.check_type('filename', filename, str)
+    #filename = isopy.checks.check_type('filename', filename, str)
     comments = isopy.checks.check_type_list('comments', comments, str, allow_none=True, make_list=True)
     keys_in = isopy.checks.check_type('keys_in', keys_in, str, allow_none=True)
     delimiter = isopy.checks.check_type('delimiter', delimiter, str)
@@ -376,7 +421,7 @@ def write_csv(filename, data, comments=None, keys_in='c',
             csize = dsize
         elif keys_in is None:
             func = _write_csv_nokeys
-            data = np.array(data.to_list())
+            data = np.array(data.to_array())
             csize = csize - 1
     else:
         ndim = data.ndim
@@ -390,31 +435,56 @@ def write_csv(filename, data, comments=None, keys_in='c',
         csize = data.shape[1] - 1
         func = _write_csv_nokeys
 
-    with open(filename, mode='w', newline='') as file:
+    rows = []
+
+    if comments is not None:
+        for comment in comments:
+            rows.append([f'{comment_symbol}{comment}'] + ['' for i in range(csize)])
+
+    rows += func(data, dsize)
+    if type(filename) is str:
+        with open(filename, mode='w', newline='') as file:
+            csv_writer = csv.writer(file, delimiter=delimiter)
+            for row in rows:
+                csv_writer.writerow(row)
+
+    if type(filename) is io.StringIO:
+        csv_writer = csv.writer(filename, delimiter=delimiter)
+        for row in rows:
+            csv_writer.writerow(row)
+
+    if type(filename) is io.BytesIO:
+        file = io.StringIO()
         csv_writer = csv.writer(file, delimiter=delimiter)
+        for row in rows:
+            csv_writer.writerow(row)
+        file.seek(0)
+        string = file.read()
+        filename.write(string.encode('UTF-8'))
 
-        if comments is not None:
-            for comment in comments:
-                csv_writer.writerow([f'{comment_symbol}{comment}'] + ['' for i in range(csize)])
-
-        func(csv_writer, data, dsize)
-
-def _write_csv_ckeys(csv_writer, data, dsize):
+def _write_csv_ckeys(data, dsize):
     keys = data.keys()
-
-    csv_writer.writerow([f'{key}' for key in keys])
+    rows = []
+    rows.append([f'{key}' for key in keys])
 
     for i in range(dsize):
-        csv_writer.writerow([f'{data[key][i]}' for key in keys])
+        rows.append([f'{data[key][i]}' for key in keys])
 
-def _write_csv_rkeys(csv_writer, data, dsize):
+    return rows
+
+def _write_csv_rkeys(data, dsize):
     keys = data.keys()
-    for key in keys:
-        csv_writer.writerow([f'{key}'] + [f'{data[key][i]}' for i in range(dsize)])
+    rows = []
 
-def _write_csv_nokeys(csv_writer, data, dsize):
+    for key in keys:
+        rows.append([f'{key}'] + [f'{data[key][i]}' for i in range(dsize)])
+    return rows
+
+def _write_csv_nokeys(data, dsize):
+    rows = []
     for i in range(data.shape[0]):
-        csv_writer.writerow([f'{data[i][j]}' for j in range(data.shape[-1])])
+        rows.append([f'{data[i][j]}' for j in range(data.shape[-1])])
+    return rows
 
 #######################
 ### Read/write xlsx ###
@@ -426,8 +496,8 @@ def read_xlsx(filename, sheetname=None, keys_in='c',
 
     Parameters
     ----------
-    filename : str
-        Path for file to be opened.
+    filename : str, bytes, BytesIO
+        Path for file to be opened. Also accepts file like objects.
     sheetname : str, int, Optional
         To load data from a single sheet in the workbook pass either the name of the sheet or
         the position of the sheet. If nothing is specified all the data for all sheets in the
@@ -446,6 +516,9 @@ def read_xlsx(filename, sheetname=None, keys_in='c',
         A dictionary containing the data for a single sheet or a dictionary containing the data
         for multiple sheets in the workbook indexed by sheet name.
     """
+    if filename is bytes:
+        filename = io.BytesIO(filename)
+
     workbook = openpyxl.load_workbook(filename, data_only=True, read_only=True)
 
     sheetnames = workbook.sheetnames
@@ -497,10 +570,12 @@ def _read_xlsx_sheet_ckeys(worksheet, row_index, default_value):
 
         for ri in range(row_index + 1, worksheet.max_row + 1):
             cell = worksheet.cell(ri, ci)
-            if cell.data_type == 'e':
+            value = cell.value
+            if type(value) is str:
+                value = value.strip()
+            if cell.data_type == 'e' or (type(value) is str and value.upper()) == '=NA()':
                 data[key].append(ERROR)
             elif cell.data_type == 's':
-                value = cell.value.strip()
                 if value == '':
                     data[key].append(None)
                 else:
@@ -536,10 +611,12 @@ def _read_xlsx_sheet_rkeys(worksheet, row_index, default_value):
 
         for ci in range(2, worksheet.max_column + 1):
             cell = worksheet.cell(ri, ci)
-            if cell.data_type == 'e':
+            value = cell.value
+            if type(value) is str:
+                value = value.strip()
+            if cell.data_type == 'e' or (type(value) is str and value.upper()) == '=NA()':
                 data[key].append(ERROR)
             elif cell.data_type == 's':
-                value = cell.value.strip()
                 if value == '':
                     data[key].append(None)
                 else:
@@ -570,10 +647,12 @@ def _read_xlsx_sheet_nokeys(worksheet, row_index, default_value):
 
         for ci in range(1, worksheet.max_column + 1):
             cell = worksheet.cell(ri, ci)
-            if cell.data_type == 'e':
+            value = cell.value
+            if type(value) is str:
+                value = value.strip()
+            if cell.data_type == 'e' or (type(value) is str and value.upper()) == '=NA()':
                 data[-1].append(ERROR)
             elif cell.data_type == 's':
-                value = cell.value.strip()
                 if value == '':
                     data[-1].append(None)
                 else:
@@ -591,15 +670,15 @@ def _read_xlsx_sheet_nokeys(worksheet, row_index, default_value):
 
 #TODO change np.nan values to '=NA()'
 def write_xlsx(filename, *sheets, comments = None, comment_symbol= '#',
-               keys_in= 'c', append = False, **sheetnames):
+               keys_in= 'c', append = False, clear = True, first_cell ="A1", **sheetnames):
     """
     Save data to an excel file.
 
     Parameters
     ----------
-    filename : str
+    filename : str, BytesIO
         Path/name of the excel file to be created. Any existing file with the same path/name
-        will be overwritten.
+        will be overwritten. Also accepts file like objects.
     sheets : isopy_array_like, numpy_array_like
         Data arrays given here will be saved as sheet1, sheet2 etc.
     comments : str, Sequence[str], Optional
@@ -612,41 +691,71 @@ def write_xlsx(filename, *sheets, comments = None, comment_symbol= '#',
     append : bool, Default = False
         If ``True`` and *filename* exists it will append the data to this workbook. An exception
         is raised if *filename* is not a valid excel workbook.
-    sheets : isopy_array, numpy_array
+    clear : bool, Default = True
+        If ``True`` any preexisting sheets are cleared before any new data is written to it.
+    first_cell: str
+        The first cell where the data is written. Defaults value is ``"A1"``.
+    sheetnames : isopy_array, numpy_array
         Data array given here will be saved in the workbook with the keyword as the sheet name
     """
-    if append and _os.path.exists(filename):
-        workbook = openpyxl.load_workbook(filename=filename)
-    else:
-        workbook = openpyxl.Workbook()
-        workbook.remove(workbook.active)
+    save = True
+    if type(filename) is openpyxl.Workbook:
+        workbook = filename
+        save = False
+    elif type(filename) is io.BytesIO:
+        if append:
+            #TODO some sort of check to make sure the file isnt empty
+            try:
+                workbook = openpyxl.load_workbook(filename=filename)
+            except:
+                workbook = openpyxl.Workbook()
+                workbook.remove(workbook.active)
+        else:
+            workbook = openpyxl.Workbook()
+            workbook.remove(workbook.active)
+    elif type(filename) is str:
+        if append and _os.path.exists(filename):
+            workbook = openpyxl.load_workbook(filename=filename)
+        else:
+            workbook = openpyxl.Workbook()
+            workbook.remove(workbook.active)
 
     sheetname_data = {f'sheet{i+1}': data for i, data in enumerate(sheets)}
     sheetname_data.update(sheetnames)
     try:
         for sheetname, data in sheetname_data.items():
-            if sheetname in workbook.sheetnames:
+            #If appending then delete any preexisting workbooks
+            if sheetname not in workbook.sheetnames:
+                worksheet = workbook.create_sheet(sheetname)
+            elif clear:
                 workbook.remove(workbook[sheetname])
+                worksheet = workbook.create_sheet(sheetname)
+            else:
+               worksheet = workbook[sheetname]
 
-            worksheet = workbook.create_sheet(sheetname)
-            _write_xlsx(worksheet, data, comments, comment_symbol, keys_in)
+            _write_xlsx(worksheet, data, comments, comment_symbol, keys_in, first_cell)
 
+        #Workbooks must have at least one sheet
         if len(workbook.sheetnames) == 0:
             workbook.create_sheet('sheet1')
 
-        workbook.save(filename)
+        if save: workbook.save(filename)
     finally:
-        workbook.close()
+        if save: workbook.close()
 
-def _write_xlsx(worksheet, data, comments, comment_symbol, keys_in):
+def _write_xlsx(worksheet, data, comments, comment_symbol, keys_in, cell):
     data = isopy.asanyarray(data)
 
-    ri = 1
+    if type(cell) is tuple:
+        ri, ci = cell
+    else:
+        ri, ci = openpyxl.utils.cell.coordinate_to_tuple(cell)
+
     if comments is not None:
         if type(comments) is not list:
             comments = [comments]
         for comment in comments:
-            worksheet.cell(ri, 1).value = f'{comment_symbol}{comment}'
+            worksheet.cell(ri, ci).value = f'{comment_symbol}{comment}'
             ri += 1
 
     if isinstance(data, isopy.core.IsopyArray):
@@ -654,13 +763,13 @@ def _write_xlsx(worksheet, data, comments, comment_symbol, keys_in):
             data = data.reshape(-1)
         if keys_in == 'c':
             data = data.to_dict()
-            _write_xlsx_ckeys(worksheet, data, ri)
+            _write_xlsx_ckeys(worksheet, data, ri, ci)
         elif keys_in == 'r':
             data = data.to_dict()
-            _write_xlsx_rkeys(worksheet, data, ri)
+            _write_xlsx_rkeys(worksheet, data, ri, ci)
         elif keys_in is None:
             data = np.array(data.to_list())
-            _write_xlsx_nokeys(worksheet, data, ri)
+            _write_xlsx_nokeys(worksheet, data, ri, ci)
     else:
         ndim = data.ndim
         if ndim == 0:
@@ -669,99 +778,55 @@ def _write_xlsx(worksheet, data, comments, comment_symbol, keys_in):
             data = data.reshape((1, -1))
         elif ndim > 2:
             raise ValueError('Cannot save data with more than two dimensions')
-        _write_xlsx_nokeys(worksheet, data, ri)
+        _write_xlsx_nokeys(worksheet, data, ri, ci)
 
 
-def _write_xlsx_ckeys(worksheet, data, ri):
-    for ci, key in enumerate(data.keys()):
-        worksheet.cell(ri, ci + 1).value = f'{key}'
+def _write_xlsx_ckeys(worksheet, data, ri, ci):
+    for cj, key in enumerate(data.keys()):
+        worksheet.cell(ri, ci + cj).value = f'{key}'
 
         for rj, value in enumerate(data[key]):
-            worksheet.cell(ri + rj + 1, ci + 1).value = value
+            if np.isnan(value):
+                value = '#N/A'
+            worksheet.cell(ri + rj + 1, ci + cj).value = value
 
-def _write_xlsx_rkeys(worksheet, data, ri):
-    for ci, key in enumerate(data.keys()):
-        worksheet.cell(ri, 1).value = f'{key}'
+def _write_xlsx_rkeys(worksheet, data, ri, ci):
+    for rj, key in enumerate(data.keys()):
+        worksheet.cell(ri + rj, ci).value = f'{key}'
 
-        for rj, value in enumerate(data[key]):
-            worksheet.cell(ri, 2 + rj).value = value
-        ri += 1
+        for cj, value in enumerate(data[key]):
+            if np.isnan(value):
+                value = '#N/A'
+            worksheet.cell(ri + rj, ci + cj + 1).value = value
 
-def _write_xlsx_nokeys(worksheet, data, ri):
-    for i, row in enumerate(data):
-        for rj, value in enumerate(row):
-            worksheet.cell(ri + i, 1 + rj).value = value
+def _write_xlsx_nokeys(worksheet, data, ri, ci):
+    for rj, row in enumerate(data):
+        for cj, value in enumerate(row):
+            if np.isnan(value):
+                value = '#N/A'
+            worksheet.cell(ri + rj, ci+ cj).value = value
 
-#################################
-### Old versions of functions ###
-#################################
-def _read_excel(filename, sheetname = None, comment_symbol='#', default_value='nan'):
+################
+### to array ###
+################
+
+def array_from_csv(filename, *, dtype=None, ndim=None, **read_csv_kwargs):
     """
-    Read data from sheets in an excel file.
-
-    The first row of the sheet will be used as the key and the subsequent rows will be taken as the value.
-
-    Parameters
-    ----------
-    filename : str
-        Location of the file to be opened.
-    sheetname : int, str, None, optional
-        If ``int`` then the sheet at this index will be returned. If ``str`` then the sheet with this name will be
-        returned. If ``None`` a dict containing the data for all the sheets in the array will be returned.
-    comment_symbol : str, optional
-        Rows at the top of each sheet starting with *comment_symbol* will be ignored. Default value is ``"#"``.
-
-    Returns
-    -------
-    dict
-        If *sheetname* is ``None`` a dict of each sheet in the file will be returned. Otherwise a dict of the data in
-        the given sheet is returned.
-
-    Examples
-    --------
-    Need to make examples
+    Returns an array of values from a csv file.
     """
-    wb = load_workbook(filename, data_only=True, read_only=True)
-    try:
-        if sheetname is None:
-            return {sheetname: _read_sheet(wb[sheetname], comment_symbol, default_value) for sheetname in wb.sheetnames}
-        elif isinstance(sheetname, int):
-            return _read_sheet(wb[wb.sheetnames[sheetname]], comment_symbol, default_value)
-        else:
-            return _read_sheet(wb.sheet_by_name(sheetname), comment_symbol, default_value)
-    finally:
-        wb.close()
+    data = read_csv(filename, **read_csv_kwargs)
+    return isopy.asanyarray(data, dtype=dtype, ndim=ndim)
 
+def array_from_xlsx(filename, sheetname, *, dtype=None, ndim=None, **read_xlsx_kwargs):
+    """
+    Returns an array of values from *sheet_name* in a xlsx file.
+    """
+    data = read_xlsx(filename, sheetname, **read_xlsx_kwargs)
+    return isopy.asanyarray(data, dtype=dtype, ndim=ndim)
 
-def _read_sheet(sheet, comment_symbol, default_value = 'nan'):
-    for ri in range(1,sheet.max_row):
-        if not ((c:=sheet.cell(ri, 1)).data_type == 's' and c.value.startswith(comment_symbol)):
-            data = {}
-            for ci in range(1, sheet.max_column+1):
-                cell = sheet.cell(ri, ci)
-                if cell.data_type == 's' and cell.value.strip() != '':
-                    column = f'{cell.value}'
-                elif cell.value is not None:
-                    column = f'column{ci}'
-                else:
-                    continue
-                data[column] = []
-                for rj in range(ri+1, sheet.max_row+1):
-                    vcell = sheet.cell(rj, ci)
-                    if vcell.data_type == 'e':
-                        data[column].append(default_value)
-                    else:
-                        data[column].append(vcell.value)
-
-            for i in range(sheet.max_row + 1 - (ri + 1)):
-                if True in (rowisnone:=[data[c][i] is None for c in data.keys()]):
-                    if set(rowisnone) == 1:
-                        #All values in row is None. In this case just remove the row
-                        for c in data.keys():
-                            data[c].pop(i)
-                    else:
-                        #Replace all None values with the default value
-                        for c in data.keys():
-                            if data[c][i] is None: data[c][i] = default_value
-
-            return data
\ No newline at end of file
+def array_from_clipboard(*, dtype=None, ndim=None, **read_clipboard_kwargs):
+    """
+    Returns an array of values from the clipboard.
+    """
+    data = read_clipboard(**read_clipboard_kwargs)
+    return isopy.asanyarray(data, dtype=dtype, ndim=ndim)
\ No newline at end of file
